@@ -7,7 +7,6 @@ from edge.models import Fragment, Fragment_Chunk_Location
 
 
 class GFFImporter(object):
-
     def __init__(self, genome, gff_fasta_fn):
         self.__genome = genome
         self.__gff_fasta_fn = gff_fasta_fn
@@ -44,18 +43,26 @@ class GFFFragmentImporter(object):
         self.parse_gff()
         t0 = time.time()
         f = self.build_fragment()
-        print('build fragment: %.4f' % (time.time() - t0,))
+        print("build fragment: %.4f" % (time.time() - t0,))
         t0 = time.time()
         self.annotate(f)
-        print('annotate: %.4f' % (time.time() - t0,))
+        print("annotate: %.4f" % (time.time() - t0,))
         return f
 
     def parse_gff(self):
-        name_fields = ('name', 'Name', 'gene', 'locus', 'locus_tag', 'product', 'protein_id')
+        name_fields = (
+            "name",
+            "Name",
+            "gene",
+            "locus",
+            "locus_tag",
+            "product",
+            "protein_id",
+        )
 
         self.__sequence = str(self.__rec.seq)
         seqlen = len(self.__sequence)
-        print('%s: %s' % (self.__rec.id, seqlen))
+        print("%s: %s" % (self.__rec.id, seqlen))
 
         features = []
         for feature in self.__rec.features:
@@ -65,7 +72,7 @@ class GFFFragmentImporter(object):
 
             # get name
             name = feature.id
-            if name == '':
+            if name == "":
                 name = feature.type
             for field in name_fields:
                 if field in feature.qualifiers:
@@ -83,16 +90,24 @@ class GFFFragmentImporter(object):
                     qualifiers[field] = v
 
             # start in Genbank format is start after, so +1 here
-            features.append((feature.location.start + 1, feature.location.end,
-                             name, feature.type, feature.strand, qualifiers))
+            features.append(
+                (
+                    feature.location.start + 1,
+                    feature.location.end,
+                    name,
+                    feature.type,
+                    feature.strand,
+                    qualifiers,
+                )
+            )
         self.__features = features
 
     def build_fragment(self):
         # pre-chunk the fragment sequence at feature start and end locations.
         # there should be no need to further divide any chunk during import.
-        break_points = list(set(
-            [f[0] for f in self.__features]
-            + [f[1] + 1 for f in self.__features]))
+        break_points = list(
+            set([f[0] for f in self.__features] + [f[1] + 1 for f in self.__features])
+        )
         break_points = sorted(break_points)
 
         cur_len = 0
@@ -110,7 +125,9 @@ class GFFFragmentImporter(object):
         if cur_len < seq_len:
             chunk_sizes.append(seq_len - cur_len)
 
-        new_fragment = Fragment(name=self.__rec.id, circular=False, parent=None, start_chunk=None)
+        new_fragment = Fragment(
+            name=self.__rec.id, circular=False, parent=None, start_chunk=None
+        )
         new_fragment.save()
         new_fragment = new_fragment.indexed_fragment()
 
@@ -129,48 +146,58 @@ class GFFFragmentImporter(object):
                     original_chunk_size -= chunk_size_limit
                 new_chunk_sizes.extend(divided_chunks)
         chunk_sizes = new_chunk_sizes
-        print('%d chunks' % (len(chunk_sizes),))
+        print("%d chunks" % (len(chunk_sizes),))
 
         prev = None
         fragment_len = 0
         for chunk_size in chunk_sizes:
             t0 = time.time()
             prev = new_fragment._append_to_fragment(
-              prev, fragment_len,
-              self.__sequence[fragment_len:fragment_len + chunk_size])
+                prev,
+                fragment_len,
+                self.__sequence[fragment_len : fragment_len + chunk_size],
+            )
             fragment_len += chunk_size
-            print('add chunk to fragment: %.4f\r' % (time.time() - t0,), end="")
+            print("add chunk to fragment: %.4f\r" % (time.time() - t0,), end="")
 
         return new_fragment
 
     def annotate(self, fragment):
-        self.__fclocs = {c.base_first: c
-                         for c in Fragment_Chunk_Location.objects
-                                                         .select_related('chunk')
-                                                         .filter(fragment=fragment)}
+        self.__fclocs = {
+            c.base_first: c
+            for c in Fragment_Chunk_Location.objects.select_related("chunk").filter(
+                fragment=fragment
+            )
+        }
 
         for feature in self.__features:
             t0 = time.time()
             f_start, f_end, f_name, f_type, f_strand, f_qualifiers = feature
             # print('  %s %s: %s-%s %s' % (f_type, f_name, f_start, f_end, f_strand))
-            self._annotate_feature(fragment, f_start, f_end, f_name, f_type, f_strand, f_qualifiers)
-            print('annotate feature: %.4f\r' % (time.time() - t0,), end="")
+            self._annotate_feature(
+                fragment, f_start, f_end, f_name, f_type, f_strand, f_qualifiers
+            )
+            print("annotate feature: %.4f\r" % (time.time() - t0,), end="")
         print("\nfinished annotating feature")
 
-    def _annotate_feature(self, fragment, first_base1, last_base1, name, type, strand, qualifiers):
+    def _annotate_feature(
+        self, fragment, first_base1, last_base1, name, type, strand, qualifiers
+    ):
         if fragment.circular and last_base1 < first_base1:
             # has to figure out the total length from last chunk
             length = len(self.__sequence) - first_base1 + 1 + last_base1
         else:
             length = last_base1 - first_base1 + 1
             if length <= 0:
-                raise Exception('Annotation must have length one or more')
+                raise Exception("Annotation must have length one or more")
 
-        if first_base1 not in self.__fclocs or\
-           (last_base1 < len(self.__sequence) and last_base1 + 1 not in self.__fclocs):
+        if first_base1 not in self.__fclocs or (
+            last_base1 < len(self.__sequence) and last_base1 + 1 not in self.__fclocs
+        ):
             raise Exception(
-                'Cannot find appropriate sequence for feature: %s, start %s, end %s' %
-                (name, first_base1, last_base1))
+                "Cannot find appropriate sequence for feature: %s, start %s, end %s"
+                % (name, first_base1, last_base1)
+            )
 
         annotation_start = self.__fclocs[first_base1]
         if last_base1 < len(self.__sequence):
@@ -187,7 +214,9 @@ class GFFFragmentImporter(object):
         a_i = 1
         while True:
             chunk = fc.chunk
-            fragment._annotate_chunk(chunk, new_feature, a_i, a_i + len(chunk.sequence) - 1)
+            fragment._annotate_chunk(
+                chunk, new_feature, a_i, a_i + len(chunk.sequence) - 1
+            )
             a_i += len(chunk.sequence)
             if fc.base_last + 1 in self.__fclocs:
                 fc = self.__fclocs[fc.base_last + 1]
